@@ -20,6 +20,8 @@ has $.plot-height       = $!height * (@!legends ?? 0.5 !! 0.65);
 
 has $.title             = '';
 
+has $.background;
+
 has &.x-tick-step       = -> $max {
     10 ** $max.log10.floor  / 2
 }
@@ -77,7 +79,7 @@ multi method plot(:$full = True, :$stacked-bars!) {
         @.eyecandy(),
     );
 
-    @.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
 }
 
 multi method plot(:$full = True, :$bars!) {
@@ -126,7 +128,7 @@ multi method plot(:$full = True, :$bars!) {
         @.eyecandy(),
     );
 
-    @.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
 }
 
 multi method plot(:$full = True, :$points!) {
@@ -163,7 +165,7 @@ multi method plot(:$full = True, :$points!) {
         @.eyecandy(),
     );
 
-    @.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
 }
 
 multi method plot(:$full = True, :$points-with-errors!,
@@ -226,7 +228,7 @@ multi method plot(:$full = True, :$points-with-errors!,
         @.eyecandy(),
     );
 
-    @.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
 }
 
 multi method plot(:$full = True, :$xy-points!) {
@@ -268,7 +270,57 @@ multi method plot(:$full = True, :$xy-points!) {
         @.eyecandy(),
     );
 
-    @.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+}
+
+multi method plot(:$full = True, :$bubbles!, :$opacity) {
+    my $label-skip = ceiling(@.values[0] / $.max-x-labels);
+
+    if $opacity.defined {
+        die "Opacity must be a value between 0 and 1"
+            unless $opacity >=0 && $opacity <= 1;
+    }
+
+    my $max_x      = [max] @.values.map: { [max] @($_[0] + $_[2]) };
+    my $min_x      = [min] @.values.map: { [min] @($_[0] - $_[2]) };
+    
+    my $max_y      = [max] @.values.map: { [max] @($_[1] + $_[2]) };
+    my $min_y      = [min] $.min-y-axis, @.values.map: { [min] @($_[1] - $_[2]) };
+
+    my $datasets   = +@.values;
+
+    my $step_x     = $.plot-width  / ($max_x - $min_x);
+    my $step_y     = $.plot-height / ($max_y - $min_y);
+
+    my @svg_d = gather {
+        for @.values[0].keys -> $k {
+            for ^$datasets -> $d {
+                my $x = @.values[$d][$k][0];
+                my $v = @.values[$d][$k][1];
+                my $r = @.values[$d][$k][2];
+                my $style = "fill:{ @.colors[$d % @.colors.elems] }";
+                $style ~= ";fill-opacity:{$opacity}" if $opacity.defined;
+                
+                my $p = 'circle' => [
+                    :cy(-($v-$min_y) * $step_y),
+                    :cx(($x - $min_x) * $step_x),
+                    :r($r * abs(1 - $min_x) * $step_x),
+                    :$style
+                ];
+                take |$.linkify($k, $p);
+            }
+        }
+
+        $.x-ticks($min_x, $max_x, $step_x);
+        $.y-ticks($min_y, $max_y, $step_y);
+    }
+
+    my $svg = $.apply-standard-transform(
+        @svg_d,
+        @.eyecandy(),
+    );
+
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
 }
 
 multi method plot(:$full = True, :$xy-lines!) {
@@ -323,7 +375,7 @@ multi method plot(:$full = True, :$xy-lines!) {
         @.eyecandy(),
     );
 
-    @.wrap-in-svg-header-if-necessary($svg, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, :wrap($full));
 }
 
 multi method plot(:$full = True, :$lines!) {
@@ -376,7 +428,7 @@ multi method plot(:$full = True, :$lines!) {
     } else {
         $lb = [];
     }
-    @.wrap-in-svg-header-if-necessary($svg, $lb, :wrap($full));
+    $.wrap-in-svg-header-if-necessary($svg, $lb, :wrap($full));
 }
 
 method y-ticks($min_y, $max_y, $scale_y, $x = 0) {
@@ -447,6 +499,15 @@ method plot-x-labels(:$label-skip, :$step_x) {
     }
 }
 
+method draw-background {
+	return unless $.background.defined;
+	'rect' => [
+		:width($.width),
+		:height($.height),
+		:style("fill: {$.background}"),
+	];
+}
+
 # plots coordinate system, title etc.
 multi method eyecandy() {
     @.plot-coordinate-system,
@@ -498,7 +559,7 @@ multi method apply-standard-transform(*@things) {
 multi method apply-coordinate-transform(*@things, :@translate) {
     return 'g' => [
         :transform("translate({@translate[0]},{@translate[1]})"),
-        @things,
+        |@things,
     ]
 }
 
@@ -508,9 +569,9 @@ method linkify($key, *@things) {
         ?? ('a' => [
                 'xlink:href' => $link,
                 :target<_top>,
-                @things
+                |@things
             ])
-        !! @things;
+        !! |@things;
 }
 
 multi method plot-legend-box() {
@@ -557,13 +618,14 @@ multi method plot-legend-box() {
 }
 
 method wrap-in-svg-header-if-necessary(*@things, :$wrap) {
+	@things.unshift: |@.draw-background;
     return $wrap
         ??
             :svg([
-                    :width($.width), :height($.height),
-                    @things
+                :width($.width), :height($.height),
+                |@things
             ])
-        !!@things;
+        !! |@things;
 }
 
 =begin Pod
